using System;
using System.Collections;
using System.ComponentModel;
using System.Data;
using System.Diagnostics;
using System.Web;
using System.Web.Services;

namespace Demo_13_SecuredService
{
	[WebService(Description="Secured Web Service",
		 Namespace="http://www.devbg.org/services/securedService/v1.0")]
	public class SecuredService : System.Web.Services.WebService
	{
		const string USER_NAME_SESSION_KEY = "UserName";

		public SecuredService()
		{
			//CODEGEN: This call is required by the ASP.NET Web Services Designer
			InitializeComponent();
		}

		#region Component Designer generated code
		
		//Required by the Web Services Designer 
		private IContainer components = null;
				
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
		}

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if(disposing && components != null)
			{
				components.Dispose();
			}
			base.Dispose(disposing);		
		}
		
		#endregion

		[WebMethod(EnableSession=true, 
			Description="Checks given credentials and establishes a session.")]
		public void Login(string aUserName, string aPassword)
		{
			if (IsLoginValid(aUserName, aPassword))
			{
				Session[USER_NAME_SESSION_KEY] = aUserName;
			}
			else
			{
				throw new AuthenticationException("Ivalid credentials!");
			}
		}

		[WebMethod(EnableSession=true, 
			Description="Terminates the active session (if any).")]
		public void Logout()
		{
			Session.Abandon();
		}

		[WebMethod(EnableSession=true, 
			Description="Returns some protected data. Requires active session.")]
		public string GetProtectedData()
		{
			CheckSecurity();
			return "This data is protected!";
		}

		[WebMethod(Description="Returns some data. Does not require a session.")]
		public string GetNotProtectedData()
		{
			return "This data is not protected.";
		}

		private void CheckSecurity()
		{
			string currentUser = (string) Session[USER_NAME_SESSION_KEY];
			if (currentUser == null)
			{
				throw new AuthenticationException(
					"Access denied! Please login first!");
			}
		}

		private bool IsLoginValid(string aUserName, string aPassword)
		{
			// Just for the demo chek if the user and pass are equal
			return (aUserName == aPassword);
		}
	}

	public class AuthenticationException : ApplicationException
	{
		public AuthenticationException(string aMessage) : base(aMessage)
		{
		}
	}
}
