using System;
using System.Collections;
using System.ComponentModel;
using System.Data;
using System.Diagnostics;
using System.Web;
using System.Web.Services;

namespace Demo_5_Service_Types
{
	[WebService(Description="Demo Web service - demonstrates complex type marshalling.",
		 Namespace="http://www.devbg.org/services/Demo_5_Service_Types/")]
	public class TypesService : System.Web.Services.WebService
	{
		public TypesService()
		{
			//CODEGEN: This call is required by the ASP.NET Web Services Designer
			InitializeComponent();
		}

		#region Component Designer generated code
		
		//Required by the Web Services Designer 
		private IContainer components = null;
				
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
		}

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if(disposing && components != null)
			{
				components.Dispose();
			}
			base.Dispose(disposing);		
		}
		
		#endregion

		[WebMethod(Description="Returns a list of available colors.")]
		public Color[] GetColors()
		{
			Color[] colors = new Color[2];
			colors[0] = Color.Blue;
			colors[1] = Color.Red;

			return colors;
		}

		[WebMethod(Description="Calculates distance between two points in the plane.")]
		public double CalculateDistance(Point p1, Point p2)
		{
			int dx = p1.x - p2.x;
			int dy = p1.y - p2.y;
			double distance = Math.Sqrt(dx*dx + dy*dy);
			return distance;
		}

		[WebMethod(Description="Converts given angle from degrees to radians.")]
		public void ConvertDegreesToRadians(ref double aAngle)
		{
			aAngle = (double) aAngle * Math.PI / 180;
		}

		[WebMethod(Description="Returns a list of Student objects.")]
		public Student[] GetStudents()
		{
			Student[] students = new Student[3];
			students[0] = new Student("", 20);
			students[1] = new Student("", 19);
			students[2] = new Student("", 21);
			return students;
		}

		[WebMethod(Description="Returns a DataSet with the tables Towns and Countries.")]
		public DataSet GetDataSet()
		{
			DataTable towns = new DataTable("Towns");
			towns.Columns.Add("id", typeof(int));
			towns.Columns.Add("name", typeof(string));
			
			DataRow sofia = towns.NewRow();
			sofia["id"] = 1;
			sofia["name"] = "";
			towns.Rows.Add(sofia);
			
			DataRow varna = towns.NewRow();
			varna["id"] = 2;
			varna["name"] = "";
			towns.Rows.Add(varna);

			DataTable countries = new DataTable("Countries");
			countries.Columns.Add("id", typeof(int));
			countries.Columns.Add("name", typeof(string));

			DataRow bg = countries.NewRow();
			bg["id"] = 1;
			bg["name"] = "";
			countries.Rows.Add(bg);

			DataSet ds = new DataSet();
			ds.Tables.Add(towns);
			ds.Tables.Add(countries);
			return ds;
		}

		[WebMethod(Description="Simultes a slow calculation.")]
		public int SlowCalculation()
		{
			System.Threading.Thread.Sleep(3000);
			return 0;
		}
	}

	public enum Color 
	{
		Red,
		Blue
	};

	public struct Point
	{
		public int x, y;
	}

	public class Student
	{
		// Private fields - not serialized
		private string mName;
		private int mAge;
		private int mState = 0;

		public string Name
		{
			get
			{
				return mName;
			}
			set
			{
				mName = value;
			}
		}

		public int Age
		{
			get
			{
				return mAge;
			}
			set
			{
				mAge = value;
			}
		}

		// This parameterless public constructor
		// is required for the XML serialization
		public Student()
		{
		}

		public Student(string aName, int aAge)
		{
			mName = aName;
			mAge = aAge;
		}
	}
}

